@@ -39,7 +39,8 @@ fn init_mathml_instance() -> RefCell<Package> {
 /// IMPORTANT: this should be the very first call to MathCAT unless the environment var MathCATRulesDir is set
 pub fn set_rules_dir(dir: String) -> Result<()> {
     use std::path::PathBuf;
-    return crate::prefs::PreferenceManager::initialize(PathBuf::from(dir));
+    let pref_manager = crate::prefs::PreferenceManager::get();
+    return pref_manager.borrow_mut().initialize(PathBuf::from(dir));
 }
 /// The MathML to be spoken, brailled, or navigated.
 /// This will override any previous MathML that was set.
@@ -174,7 +175,9 @@ pub fn set_preference(name: String, value: String) -> Result<()> {
                     let mut pref_manager = rules.pref_manager.borrow_mut();
                     files_changed = pref_manager.set_user_prefs("SpeechStyle", &value);
                 };
-                rules.invalidate(files_changed);
+                if let Some(files_changed) = files_changed {
+                    rules.invalidate(files_changed);
+                }
             },
             "Language" => {
                 // check the format
@@ -183,13 +186,17 @@ pub fn set_preference(name: String, value: String) -> Result<()> {
                         bail!("Improper format for 'Language' preference '{}'. Should be of form 'en' or 'en-gb'", value);
                       }
                 let files_changed = rules.pref_manager.borrow_mut().set_user_prefs(&name, &value);  
-                rules.invalidate(files_changed);  
+                if let Some(files_changed) = files_changed {
+                    rules.invalidate(files_changed);
+                }
             },
             "BrailleCode" => {
                 let files_changed = rules.pref_manager.borrow_mut().set_user_prefs(&name, &value);    
                 crate::speech::BRAILLE_RULES.with(|braille_rules| {
-                    braille_rules.borrow_mut().invalidate(files_changed);
-                })
+                    if let Some(files_changed) = files_changed {
+                        braille_rules.borrow_mut().invalidate(files_changed);
+                    }
+                   })
             },
             "Pitch" | "Rate" | "Volume" => {
                 rules.pref_manager.borrow_mut().set_api_float_pref(&name, to_float(&name, value)?);    
