@@ -313,6 +313,7 @@ impl fmt::Display for PreferenceManager {
     }
 }
 
+#[derive(Default)]
 pub struct FilesChanged {
     pub speech_rules: bool,
     pub speech_unicode_short: bool,
@@ -323,20 +324,17 @@ pub struct FilesChanged {
     pub defs: bool
 }
 
+impl fmt::Display for FilesChanged {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        writeln!(f, "FilesChanged {{\n  Speech: rules {}, short {}, full {}", self.speech_rules, self.speech_unicode_short, self.speech_unicode_full)?;
+        writeln!(f, "  Braille: rules {}, short {}, full {}", self.braille_rules, self.braille_unicode_short, self.braille_unicode_full)?;
+        writeln!(f, "  Defs {}", self.defs)?;
+        return Ok(());
+    }
+}
+
 impl FilesChanged {
     // FIX: this should include all files
-    fn none() -> FilesChanged {
-        return FilesChanged {
-            speech_rules: false,
-            speech_unicode_short: false,
-            speech_unicode_full: false,
-            braille_rules: false,
-            braille_unicode_short: false,
-            braille_unicode_full: false,
-            defs: false
-        }
-    }
-
     pub fn add_changes(&mut self, additional_changes: FilesChanged) {
         self.speech_rules |= additional_changes.speech_rules;
         self.speech_unicode_short |= additional_changes.speech_unicode_short;
@@ -351,27 +349,24 @@ impl FilesChanged {
 impl PreferenceManager {
     /// Initialize (the) PreferenceManager (a global var).
     /// 'rules_dir' is the path to "Rules" unless the env var MathCATRulesDir is set
-    pub fn initialize(rules_dir: PathBuf) -> Result<()> {
+    pub fn initialize(&mut self, rules_dir: PathBuf) -> Result<()> {
         // first, read in the preferences -- need to determine which files to read next
         // the prefs files are in the rules dir and the user dir; differs from other files
-        return PREF_MANAGER.with( |pref_manager| {
-            let mut pref_manager = pref_manager.borrow_mut();
-            pref_manager.api_prefs = Preferences{ prefs: DEFAULT_API_PREFERENCES.with(|defaults| defaults.prefs.clone()) };
+        self.api_prefs = Preferences{ prefs: DEFAULT_API_PREFERENCES.with(|defaults| defaults.prefs.clone()) };
 
-            match PreferenceManager::find_rules_dir(&rules_dir) {
-                Ok(rules_dir) => {
-                    let (user_prefs, pref_files) = Preferences::from_file(&rules_dir)?;
-                    match pref_manager.set_all_files(&rules_dir, user_prefs, pref_files) {
-                        Ok(_) => return Ok(()),
-                        Err(e) => pref_manager.error = errors_to_string(&e),
-                    }
-                },
-                Err(e) => {
-                    pref_manager.error = errors_to_string(&e);
-                },
-            };
-            bail!("{}", pref_manager.error);
-        })
+        match PreferenceManager::find_rules_dir(&rules_dir) {
+            Ok(rules_dir) => {
+                let (user_prefs, pref_files) = Preferences::from_file(&rules_dir)?;
+                match self.set_all_files(&rules_dir, user_prefs, pref_files) {
+                    Ok(_) => return Ok(()),
+                    Err(e) => self.error = errors_to_string(&e),
+                }
+            },
+            Err(e) => {
+                self.error = errors_to_string(&e);
+            },
+        };
+        bail!("{}", self.error);
     }
 
     pub fn get() -> Rc<RefCell<PreferenceManager>> {
@@ -437,11 +432,24 @@ impl PreferenceManager {
 
 
     fn get_file_and_time(rules_dir: &PathBuf, lang: &str, default_lang: Option<&str>, file_name: &str) -> Result<FileAndTime> {
+        use std::fs;
         let files = PreferenceManager::get_files(rules_dir, lang, default_lang, file_name)?;
         return Ok(FileAndTime {
-            time: if cfg!(target_family = "wasm") {None} else {Some( SystemTime::now() )},
+            time: if cfg!(target_family = "wasm") {None} else {get_metadata(&files[0])},
             files
-        })
+        });
+
+        fn get_metadata(path: &Option<PathBuf>) -> Option<SystemTime> {
+            if let Some(path) = path {
+                let metadata = fs::metadata(path);
+                if let Ok(metadata) = metadata {
+                    if let Ok(mod_time) = metadata.modified() {
+                        return Some(mod_time);
+                    }
+                }
+            }
+            return None;
+        }
     }
 
    fn get_files(rules_dir: &PathBuf, lang: &str, default_lang: Option<&str>, file_name: &str) -> Result<Locations> {
@@ -545,14 +553,52 @@ impl PreferenceManager {
                     &bad_env_value, rules_dir.to_str().unwrap_or("rules dir is none???"));
     }
 
-    pub fn is_up_to_date(&self) -> bool {
+    pub fn is_up_to_date(&mut self) -> Option<FilesChanged> {
         // this will work even if self is invalid
-        return PreferenceManager::is_file_up_to_date(&self.pref_files) &&
-               PreferenceManager::is_file_up_to_date(&self.speech) &&
-               PreferenceManager::is_file_up_to_date(&self.speech_unicode) &&
-               PreferenceManager::is_file_up_to_date(&self.braille) &&
-               PreferenceManager::is_file_up_to_date(&self.braille_unicode) &&
-               PreferenceManager::is_file_up_to_date(&self.defs) ;
+        let mut files_changed = FilesChanged {
+            speech_rules: PreferenceManager::is_file_up_to_date(&self.speech),
+            speech_unicode_short: PreferenceManager::is_file_up_to_date(&self.speech_unicode),
+            speech_unicode_full: PreferenceManager::is_file_up_to_date(&self.speech_unicode_full),
+            braille_rules: PreferenceManager::is_file_up_to_date(&self.braille),
+            braille_unicode_short: PreferenceManager::is_file_up_to_date(&self.braille_unicode),
+            braille_unicode_full: PreferenceManager::is_file_up_to_date(&self.braille_unicode_full),
+            defs: PreferenceManager::is_file_up_to_date(&self.defs),
+        };
+
+        if !PreferenceManager::is_file_up_to_date(&self.pref_files) {
+            let old_lang = self.user_prefs.to_string("Language");
+            let old_speech_style = self.user_prefs.to_string("SpeechStyle");
+            let old_braille_code = self.user_prefs.to_string("BrailleCode");
+            match self.initialize(self.rules_dir.clone().unwrap()) {
+                Err(e) => error!("Failed to reread prefs.yaml: {}", e),  // probably in big trouble, but continue on and maybe ok
+                Ok(_) => {
+                    if old_speech_style != self.user_prefs.to_string("SpeechStyle") {
+                        files_changed.speech_rules = false;
+                    }
+                    if old_lang != self.user_prefs.to_string("Language") {
+                        files_changed.speech_rules = false;
+                        files_changed.speech_unicode_short = false;
+                        files_changed.speech_unicode_full = false;
+                    }
+                    if old_braille_code != self.user_prefs.to_string("BrailleCode") {
+                        files_changed.braille_rules = false;
+                        files_changed.braille_unicode_short = false;
+                        files_changed.braille_unicode_full = false;
+                    }
+                }
+            } 
+        }
+        if files_changed.speech_rules &&
+           files_changed.speech_unicode_short &&
+           files_changed.speech_unicode_full &&
+           files_changed.braille_rules &&
+           files_changed.braille_unicode_short &&
+           files_changed.braille_unicode_full &&
+           files_changed.defs {
+            return None;
+        } else {
+            return Some(files_changed);
+        }
     }
 
     fn is_file_up_to_date(ft: &FileAndTime) -> bool {
@@ -709,7 +755,7 @@ impl PreferenceManager {
     //     return yaml_to_string(self.user_prefs.prefs.get(pref_name).unwrap(), 1);
     // }
 
-    pub fn set_user_prefs(&mut self, name: &str, value: &str) -> FilesChanged {
+    pub fn set_user_prefs(&mut self, name: &str, value: &str) -> Option<FilesChanged> {
         if !self.error.is_empty() {
             panic!("Internal error: set_user_prefs called on invalid PreferenceManager -- error message\n{}", &self.error);
         };
@@ -735,10 +781,10 @@ impl PreferenceManager {
                     braille_unicode_full: old_braille_unicode_full != self.braille_unicode_full,
                     defs: old_defs != self.defs,
                 };
-                return changed;
+                return Some(changed);
             }
         }
-        return FilesChanged::none();
+        return None;
     }
 }
 
@@ -775,18 +821,18 @@ mod tests {
 
     #[test]
     fn find_simple_style() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
-            let pref_manager = pref_manager.borrow_mut();
+            let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             assert_eq!(rel_path(&pref_manager.rules_dir, &pref_manager.speech.files[0]), PathBuf::from("Languages/en/ClearSpeak_Rules.yaml"));
         });
     }
 
     #[test]
     fn find_style_other_language() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz");
             assert_eq!(rel_path(&pref_manager.rules_dir, &pref_manager.speech.files[0]), PathBuf::from("Languages/zz/ClearSpeak_Rules.yaml"));
         });
@@ -794,9 +840,9 @@ mod tests {
 
     #[test]
     fn find_unicode_files() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-aa");
             
             assert_eq!(rel_path(&pref_manager.rules_dir, &pref_manager.speech.files[0]), PathBuf::from("Languages/zz/ClearSpeak_Rules.yaml"));
@@ -806,9 +852,9 @@ mod tests {
 
     #[test]
     fn find_style_no_sublanguage() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-ab");
             
             assert_eq!(rel_path(&pref_manager.rules_dir, &pref_manager.speech.files[0]), PathBuf::from("Languages/zz/ClearSpeak_Rules.yaml"));
@@ -825,9 +871,9 @@ mod tests {
                     file_name, count, correct);
         }
 
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-aa");
             pref_manager.set_user_prefs("BrailleCode", "UEB");
             
@@ -848,9 +894,9 @@ mod tests {
 
     #[test]
     fn file_found_order() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-aa");
 
             let mut iter = pref_manager.defs.files.iter();
@@ -863,9 +909,9 @@ mod tests {
 
     #[test]
     fn test_prefs() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
-            let pref_manager = pref_manager.borrow();
+            let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             let prefs = pref_manager.get_user_prefs();
             assert_eq!(prefs.to_string("Language").as_str(), "en");
             assert_eq!(prefs.to_string("SubjectArea").as_str(), "General");
@@ -878,9 +924,9 @@ mod tests {
 
     #[test]
     fn test_language_change() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("SpeechStyle", "ClearSpeak");
 
             assert_eq!(rel_path(&pref_manager.rules_dir, &pref_manager.get_rule_file(&RulesFor::Speech)[0]), PathBuf::from("Languages/en/ClearSpeak_Rules.yaml"));
@@ -893,9 +939,9 @@ mod tests {
 
     #[test]
     fn test_some_changes() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Verbosity", "Terse");
 
             assert_eq!(&pref_manager.get_user_prefs().to_string("Verbosity"), "Terse");
@@ -908,11 +954,11 @@ mod tests {
 
     #[test]
     fn test_is_up_to_date() {
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-ab");        
-            assert!(pref_manager.is_up_to_date());
+            assert!(pref_manager.is_up_to_date().is_none());
         });
     }
 
@@ -921,11 +967,10 @@ mod tests {
     fn test_is_not_up_to_date() {
         use std::thread::sleep;
         use std::time::Duration;
-        PreferenceManager::initialize(abs_rules_dir_path()).unwrap();
         PREF_MANAGER.with(|pref_manager| {
             let mut pref_manager = pref_manager.borrow_mut();
+            pref_manager.initialize(abs_rules_dir_path()).unwrap();
             pref_manager.set_user_prefs("Language", "zz-aa");        
-            sleep(Duration::from_millis(10));
 
             // Note: need to use pattern match to avoid borrow problem
             if let Some(file_name) = &pref_manager.get_rule_file(&RulesFor::Speech)[0] {
@@ -933,8 +978,11 @@ mod tests {
                 let contents = fs::read(file_name).expect(&format!("Failed to write file {} during test", file_name_as_str));
                 #[allow(unused_must_use)] { 
                     fs::write(file_name, contents);
+                    sleep(Duration::from_millis(10));
                 }
-                assert!(!pref_manager.is_up_to_date());    
+                let files_changed = pref_manager.is_up_to_date();
+                assert!(files_changed.is_some());
+                assert!(!files_changed.unwrap().speech_rules);
             } else {
                 panic!("First path is 'None'");
             }
